﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;
using System.Linq.Dynamic;

namespace Dynamic
{
    public class Pracownik
    {
        public int PracownikId { get; set; }
        public string Login { get; set; }
        public string Imie { get; set; }
        public string Nazwisko { get; set; }
        public DateTime DataZatrudnienia { get; set; }
        public Oddzial Oddzial { get; set; }
    }

    public class Oddzial
    {
        public Oddzial()
        {
            Pracownicy = new List<Pracownik>();
        }
        public int OddzialId { get; set; }
        public string Nazwa { get; set; }
        public IList<Pracownik> Pracownicy { get; set; }
    }

    public class Repository
    {
        List<Pracownik> pracownicy = new List<Pracownik>();
        List<Oddzial> oddzialy = new List<Oddzial>();

        public Repository()
        {
            var o1 = new Oddzial { OddzialId = 1, Nazwa = "Marketing" };
            var o2 = new Oddzial { OddzialId = 2, Nazwa = "Handlowy" };
            var o3 = new Oddzial { OddzialId = 3, Nazwa = "Produkcja" };

            var p1 = new Pracownik { PracownikId = 1, Login = "M01", Imie = "J1", Nazwisko = "K1", DataZatrudnienia = new DateTime(2009, 1, 11), Oddzial = o1 };
            var p2 = new Pracownik { PracownikId = 2, Login = "M02", Imie = "J2", Nazwisko = "K2", DataZatrudnienia = new DateTime(2011, 2, 11), Oddzial = o1 };
            var p3 = new Pracownik { PracownikId = 3, Login = "X01", Imie = "J1", Nazwisko = "K3", DataZatrudnienia = new DateTime(2012, 3, 11), Oddzial = o2 };
            var p4 = new Pracownik { PracownikId = 4, Login = "X02", Imie = "J2", Nazwisko = "L4", DataZatrudnienia = new DateTime(2013, 4, 11), Oddzial = o2 };

            pracownicy.Add(p1);
            pracownicy.Add(p2);
            pracownicy.Add(p3);
            pracownicy.Add(p4);

            oddzialy.Add(o1);
            oddzialy.Add(o2);
            oddzialy.Add(o3);

            o1.Pracownicy.Add(p1);
            o1.Pracownicy.Add(p2);
            o2.Pracownicy.Add(p3);
            o2.Pracownicy.Add(p4);
        }

        public List<Pracownik> GetPracownicy()
        {
            return this.pracownicy;
        }

        public List<Oddzial> GetOddzialy()
        {
            return this.oddzialy;
        }
    }

    class Program
    {
        static void Main(string[] args)
        {
            var r = new Repository();

            // statyczne
            Expression<Func<Pracownik, string>> orderExpression = e => e.Nazwisko;
            var p1List = r.GetPracownicy().OrderBy(orderExpression.Compile());

            // dynamic
            string poleSortowania = "Nazwisko";

            var parameter = Expression.Parameter(typeof(Pracownik), "Nazwisko");
            var getter = Expression.Property(parameter, typeof(Pracownik).GetProperty(poleSortowania));
            var lambda = Expression.Lambda<Func<Pracownik,string>>(getter, parameter);

            var p2List = r.GetPracownicy().OrderByDescending(lambda.Compile());

            // dynamic2
            var p3List = r.GetPracownicy().AsQueryable()
                .Where("Oddzial.OddzialId = 1")
                .OrderBy("Nazwisko");

        }
    }
}
